# XDAQ is a special case
#
# find_ups_xdaq(  minimum )
#  minimum - minimum version required

include(CheckUpsVersion)
include(CMakeParseArguments)

function(_set_xdaq_lib_vars)
  cmake_parse_arguments(SRLV "OPTIONAL" "" "" ${ARGN})
  foreach (XDAQLIB ${ARGN})
    string(TOUPPER ${XDAQLIB} XDAQLIB_UC)
    if (EXISTS ${XDAQ_FQ_DIR}/lib/lib${XDAQLIB}.so)
      set(XDAQ_${XDAQLIB_UC} ${XDAQ_FQ_DIR}/lib/lib${XDAQLIB}.so PARENT_SCOPE)
    elseif (NOT SRLV_OPTIONAL)
      message(SEND_ERROR "find_ups_xdaq: expected XDAQ library lib${XDAQLIB}.so missing.")
    endif()
  endforeach()
endfunction()

function(_set_and_check_prog VAR PROG)
  if (NOT EXISTS ${PROG})
    message(SEND_ERROR "find_ups_xdaq: expected XDAQ program ${PROG} missing.")
  endif()
  set(${VAR} ${PROG} PARENT_SCOPE)
endfunction()

# since variables are passed, this is implemented as a macro
macro( find_ups_xdaq minimum )

# require XDAQ_FQ_DIR
set( XDAQ_FQ_DIR $ENV{XDAQ_FQ_DIR} )
if ( NOT XDAQ_FQ_DIR )
  message(FATAL_ERROR "xdaq has not been setup")
endif ()

# only execute if this macro has not already been called
if( NOT XDAQ_VERSION )
  ##message( STATUS "find_ups_xdaq debug: XDAQ_VERSION is NOT defined" )

SET ( XDAQ_STRING $ENV{SETUP_XDAQ} )
set( XDAQ_VERSION $ENV{XDAQ_VERSION} )
if ( NOT XDAQ_VERSION )
   #message( STATUS "find_ups_xdaq: calculating xdaq version" )
   STRING( REGEX REPLACE "^[r][o][o][t][ ]+([^ ]+).*" "\\1" XDAQ_VERSION "${XDAQ_STRING}" )
endif ()

#message( STATUS "find_ups_xdaq: checking xdaq ${XDAQ_VERSION} against ${minimum}" )
_check_version( XDAQ ${XDAQ_VERSION} ${minimum} )
set( XDAQ_DOT_VERSION ${dotver} )
# compare for recursion
list(FIND cet_product_list xdaq found_product_match)
if( ${found_product_match} LESS 0 )
  # add to product list
  set(CONFIG_FIND_UPS_COMMANDS "${CONFIG_FIND_UPS_COMMANDS}
    find_ups_xdaq( ${minimum} )")
  set(cet_product_list xdaq ${cet_product_list} )
endif()

STRING( REGEX MATCH "[-][q]" has_qual  "${XDAQ_STRING}" )
STRING( REGEX MATCH "[-][j]" has_j  "${XDAQ_STRING}" )
if( has_qual )
  if( has_j )
     STRING( REGEX REPLACE ".*([-][q]+ )(.*)[ *]([-][-j])" "\\2" XDAQ_QUAL "${XDAQ_STRING}" )
  else( )
     STRING( REGEX REPLACE ".*([-][q]+ )(.*)" "\\2" XDAQ_QUAL "${XDAQ_STRING}" )
  endif( )
  STRING( REGEX REPLACE ":" ";" XDAQ_QUAL_LIST "${XDAQ_QUAL}" )
  list(REMOVE_ITEM XDAQ_QUAL_LIST debug opt prof)
  STRING( REGEX REPLACE ";" ":" XDAQ_BASE_QUAL "${XDAQ_QUAL_LIST}" )
else( )
  message(STATUS "XDAQ has no qualifier")
endif( )
if( ${found_product_match} LESS 0 )
  _cet_debug_message("find_ups_xdaq: XDAQ version and qualifier are ${XDAQ_VERSION} ${XDAQ_QUAL}" )
endif()
#message(STATUS "XDAQ base qualifier is ${XDAQ_BASE_QUAL}" )

# add include directory to include path if it exists
include_directories ( $ENV{XDAQ_INC} )
include_directories ( $ENV{XDAQ_PLAT_INC} )

# Removed from v13_0_0 package: frl2g  ptfrl sentinelprobe2g sentinelspotlight 
# sqlite3 wsbroker wsbrokerutils xmem  xmassensor2g xmaslas xmaspulser
# wsclientsubscriber wseventing wsserviceeventing wsutils xmassensor
# jansson ssh ssh_threads xalan-c xalanMsg
# Removed from v14_0_0 package: 
# espai esxbeat esxtreme eventingapi eventingcore gevb2g mstreamio2g
# ptblitptibv pttcp ptudapl ptutcp roundtrip
# sentinelbridge2g sentinelprobe sentinelsentineld
# sentinel sentinelspotlight2g sentineltester
# sentinelutils tcpla timeline tracerd tracerprobe
# wsaddressing xaccess xmasadmin xmasbridge2g
# xmasbroker2g xmascollector2g
# xmasheartbeatheartbeatd xmasheartbeatprobe xmasheartbeat
# xmaslas2g xmasprobe
# xmassensord xmasslash2g xmassmarthub
# xmastester xmasutils xmemprobe xplore xploreutils xslp

# define XDAQ libraries
_set_xdaq_lib_vars(
b2ineventing b2innub b2inutils cgicc config   
executive  hyperdaq i2o log4cplus logudpappender 
logxmlappender mimetic peer ptfifo pthttp toolbox 
xcept xdaq xdata xgi xoapfilter xoap xrelay
)

# define xdaqcint executable
_set_and_check_prog(XDAQEXE ${XDAQ_FQ_DIR}/bin/xdaq.exe)

# define some useful library lists
set(XDAQ_BASIC_LIB_LIST ${XDAQ_XDAQ}
                        ${XDAQ_XDATA} 
                        ${XDAQ_XCEPT}
                        ${XDAQ_EXECUTIVE}
                        ${XDAQ_CONFIG}
                        ${XDAQ_CGICC}
			${XDAQ_XALAN-C}
			${XDAQ_XOAP}
                        ${XDAQ_HYPERDAQ}
			${XDAQ_TOOLBOX}
			dl
			pthread
)
endif( NOT XDAQ_VERSION )

endmacro( find_ups_xdaq )
