// $Id$

/*************************************************************************
 * XDAQ Components for Distributed Data Acquisition                      *
 * Copyright (C) 2000-2014, CERN.                                        *
 * All rights reserved.                                                  *
 * Authors: L.Orsini, A.Petrucci, A.Forrest								 *
 * For the licensing terms see LICENSE.                                  *
 * For the list of contributors see CREDITS.                             *
 *************************************************************************/

#ifndef _pt_ibv_Address_h
#define _pt_ibv_Address_h

#include "pt/Address.h"
#include "toolbox/net/URL.h"
#include <netinet/in.h>

namespace pt
{
	namespace ibv
	{
		class Address : public pt::Address
		{
			public:

				//! Create address from url
				Address (const std::string & url, const std::string& service, size_t ibPort, size_t ibPath);

				virtual ~Address ();

				//! Get the URL protocol, e.g. ptdapl
				std::string getProtocol ();

				//! Get the URL service if it exists, e.g. i2o
				std::string getService ();

				//! Get additional parameters at the end of the URL
				std::string getServiceParameters ();

				//! Return the URL in string form
				std::string toString ();

				//! Get the host part of the url
				std::string getHost ();

				//! Get the port number of the url if it exists
				std::string getPort ();

				//! Get the IB port number
				size_t getIBPort ();

				//! Get the IB path (src_path_bits) for use with HCA LMC > 0
				size_t getIBPath ();

				//! Get the port number of the url if it exists
				unsigned short getPortNum ();

				//! Get the url provided in the constructor
				std::string getURL ();

				//! Get the path
				std::string getPath ();

				//! Compare with another address
				bool equals (pt::Address::Reference address);

			protected:

				toolbox::net::URL url_;
				std::string service_;

				size_t ibPort_;
				size_t ibPath_;
		};
	}
}

#endif
